/***************************************************************************
 *   Copyright (C) 2004 by EVER Sp. z o.o.                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

extern "C" {
#include <pthread.h>
}
#include <signal.h>
#include "monitor.h"
#include "map.h"
#include "upsints.h"

LPFNIOCTL		drvfn=NULL;		// driver function pointer
sflow_ctl		afctl;			// main application control struct (contains flags)
sapp_config		appcfg;			// application configuration structure
suser_config	usrcfg;			// user configuration structure
CLog		evtlog;
CConf		*pCfgMain=NULL, *pCfgUser=NULL, *pCfgDev=NULL;
CShMem		*pShmMain=NULL, *pShmUser=NULL;
pthread_t	pthMonitor, pthAction, pthShutdown;

const int iShmSize = PS_SHM_SIZE;

bool	bUpsShutdownNow=false;		// flag to perform ups shutdown
bool	bUpsStateReadingFlag=false;	// flag identifying that data is being read from UPS
bool	bShutdownTmActive=false;	// shutdown thread is active
bool	bNetShutdownTmActive=false;	// network shutdown thread is active
bool	bDrvInitialized=false;		// is driver initialized?
bool	bUpsOperationFlag=false;	// if any operation on UPS is currently performed

#undef ISSETAPPFLAG
#undef SETAPPFLAG
#undef CLRAPPFLAG
#define ISSETAPPFLAG(f)		(flag_check(appcfg.ulAppFlag, f))
#define SETAPPFLAG(f)		(flag_set(&appcfg.ulAppFlag, f))
#define CLRAPPFLAG(f)		(flag_clear(&appcfg.ulAppFlag, f))

#define GETAPPFLOWCTL		(pShmMain->read(&afctl, AFLOWCTL_OFFSET, sizeof(sflow_ctl), true))
#define UPDATEAPPFLOWCTL	(pShmMain->write(&afctl, AFLOWCTL_OFFSET, sizeof(sflow_ctl), true))
#define GETAPPCFG			(pShmMain->read(&appcfg, APPCFG_OFFSET, sizeof(sapp_config), true))
#define UPDATEAPPCFG		(pShmMain->write(&appcfg, APPCFG_OFFSET, sizeof(sapp_config), true))
#define GETUSRCFG			(pShmMain->read(&usrcfg, USRCFG_OFFSET, sizeof(suser_config), true))
#define UPDATEUSRCFG		(pShmMain->write(&usrcfg, USRCFG_OFFSET, sizeof(suser_config), true))


/*
 * returns pointer to specified ups interface function
 */
LPFNIOCTL get_ups_interface(char *name)
{
	int ic=0;
	if (name==NULL)
		return NULL;
	while(ups_interface[ic].ups_name!=0) {
		if (strcasecmp(ups_interface[ic].ups_name, name)==0) {
			return (LPFNIOCTL)(ups_interface[ic].fnctl);
		}
		ic++;
	}
	return NULL;
}

/*
 * Gather all information from
 */
int getAllUpsData()
{
#undef SUPPORTED
#define SUPPORTED(d)	(flag_check(appcfg.supsi.ulMask, d))

	unsigned long ulBuff;
	int iReturn;
	if (drvfn==NULL)
		return 0;

	// Connection test
	iReturn=drvfn(IOCTL_TESTUPSLINK2, NULL, NULL);
	if (iReturn!=IOCTL_ERROR_CMD_NOTSUPPORTED)
	{
		if (iReturn!=IOCTL_ERROR_SUCCESS)
		{
			appcfg.iErrorCounter++;
			return 0;
		}
	}

	// UPS shutdown
	if (afctl.bUpsShutdownNow)
	{
		iReturn=drvfn(IOCTL_SHUTDOWNUPS, NULL, NULL);
		if (iReturn!=IOCTL_ERROR_SUCCESS)
		{
			evtlog.new_msg(msg_error, TXT_ERR_UPSSHUTDOWN, iReturn);
			GETAPPFLOWCTL;
			afctl.bUpsShutdownNow=false;
			UPDATEAPPFLOWCTL;
			appcfg.iErrorCounter++;
		}
		else
		{
			evtlog.new_msg(msg_info, TXT_INF_UPSSHUTDOWN);
			sleep(1);
			GETAPPFLOWCTL;
			afctl.bUpsShutdownNow=false;
			UPDATEAPPFLOWCTL;
			return -1;	// now leave, because we don't want to get Low Battery status
		}
	}
	// update configuration
	if (afctl.bConfigUpdate)
	{
		iReturn=drvfn(IOCTL_UPDATECONFIG, NULL, NULL);
		if (iReturn!=IOCTL_ERROR_SUCCESS)
		{
			iReturn=drvfn(IOCTL_GET_ERRORNO, NULL, NULL);
			evtlog.new_msg(msg_error, TXT_ERR_UPDATECONFIG, iReturn);
			appcfg.iErrorCounter++;
			return 0;
		}
		else
		{
			evtlog.new_msg(msg_info, TXT_INF_UPDATECONFIG, iReturn);
			GETAPPFLOWCTL;
			afctl.bConfigUpdate=false;
			UPDATEAPPFLOWCTL;
		}
	}
	if (SUPPORTED(UI_UPS_STATE))
	{
		GETAPPFLOWCTL;
		afctl.bUpsStateReading=true;
		UPDATEAPPFLOWCTL;
		unsigned long ulBuff=sizeof(appcfg.supsi.ulUpsStateMask);
		iReturn=drvfn(IOCTL_GET_UPSSTATESMASK, 
			(void *)&appcfg.supsi.ulUpsStateMask, (int *)&ulBuff);
		if (iReturn!=IOCTL_ERROR_SUCCESS)
		{
#ifdef _DEBUGMODE			
			evtlog.new_msg(msg_error, TXT_ERR_GETUPSSTATE, iReturn);
#endif//_DEBUGMODE
			appcfg.iErrorCounter++;
			GETAPPFLOWCTL;
			afctl.bUpsStateReading=false;
			UPDATEAPPFLOWCTL;
			return 0;
		}
		else
		{
			unsigned long ulBuff=sizeof(appcfg.supsi.ulUpsState);
			iReturn=drvfn(IOCTL_GET_UPSSTATE, 
				(void *)&appcfg.supsi.ulUpsState, (int *)&ulBuff);
			if (iReturn!=IOCTL_ERROR_SUCCESS)
			{
#ifdef _DEBUGMODE			
				evtlog.new_msg(msg_error, TXT_ERR_GETUPSSTATE, iReturn);
#endif//_DEBUGMODE
				appcfg.iErrorCounter++;
				GETAPPFLOWCTL;
				afctl.bUpsStateReading=false;
				UPDATEAPPFLOWCTL;
				return 0;
			}
		}
		GETAPPFLOWCTL;
		bUpsStateReadingFlag=false;
		UPDATEAPPFLOWCTL;
	}
	if (SUPPORTED(UI_PARAMETERS))
	{
		unsigned long ulBuff=sizeof(appcfg.supsi.sUpsParameters);
		iReturn=drvfn(IOCTL_GET_UPSPARAMS, 
			(void *)&appcfg.supsi.sUpsParameters, (int *)&ulBuff);
		if (iReturn!=IOCTL_ERROR_SUCCESS)
		{	
#ifdef _DEBUGMODE			
			evtlog.new_msg(msg_error, TXT_ERR_GETUPSPARAMS, iReturn);
#endif//_DEBUGMODE
			appcfg.iErrorCounter++;
			return 0;
		}
	}
	if (SUPPORTED(UI_SETUP_PARAMETERS))
	{
		unsigned long ulBuff=sizeof(appcfg.supsi.sUpsSetupParameters);
		iReturn=drvfn(IOCTL_GET_UPSSETUPPARAMS, 
			(void *)&appcfg.supsi.sUpsSetupParameters, (int *)&ulBuff);
		if (iReturn!=IOCTL_ERROR_SUCCESS)
		{
#ifdef _DEBUGMODE
			evtlog.new_msg(msg_error, TXT_ERR_GETUPSSETUPPARAMS, iReturn);
#endif//_DEBUGMODE
			appcfg.iErrorCounter++;
			return 0;
		}
	}
	return 1;
}

int checkUpsStates()
{
#undef SUPPORTED
#undef NOTPASSED
#undef PASSED
#undef ISSET
#undef SETACT
#undef SETPASS
#undef ALLOK
#undef RESETIFNOMORESET
#undef IFNOMORESET
#define SUPPORTED(d)	(flag_check(appcfg.supsi.ulUpsStateMask, d))
#define NOTPASSED(d)	(!flag_check(appcfg.ulOpFlag, d))
#define PASSED(d)		(flag_check(appcfg.ulOpFlag, d))
#define ISSET(d)		(flag_check(appcfg.supsi.ulUpsState, d))
#define SETACT(d)		(flag_set(&appcfg.ulActFlag, d))
#define SETPASS(d)	(flag_set(&appcfg.ulOpFlag, d))
#define CLEARACT(d)	(flag_clear(&appcfg.ulActFlag, d))
#define ALLOK(d1, d2, d3)	(SUPPORTED(d1) && NOTPASSED(d2) && ISSET(d3))
#define RESETIFNOMORESET(d1, d2, d3) {														\
	if (SUPPORTED(d1) && PASSED(d2) && !ISSET(d3)) 										\
		flag_clear(&appcfg.ulOpFlag, d2); }
#define IFNOMORESET(d1, d2, d3) (SUPPORTED(d1) && PASSED(d2) && !ISSET(d3))

	// Online work 
	if (ALLOK(US_POWERON, OF_POWERON_PASS, US_POWERON)) {
		flag_clear(&appcfg.ulOpFlag, 
			OF_POWERFAIL_PASS | OF_STANDBY_PASS | OF_WATCH_PASS | OF_BATTERYLOW_PASS);
		SETPASS(OF_POWERON_PASS);
		SETACT(ACF_POWERON);
		evtlog.new_msg(msg_info, TXT_INF_POWERON);
		/* wylaczamy flage aby przerwac ewentualny shutdown */
		GETAPPFLOWCTL;
		afctl.bShutdownProcActive=afctl.bNetShutdownProcActive=false;
		UPDATEAPPFLOWCTL;
	}
	// Battery work
	if (ALLOK(US_POWERFAIL, OF_POWERFAIL_PASS, US_POWERFAIL)) {
		flag_clear(&appcfg.ulOpFlag, OF_POWERON_PASS | OF_STANDBY_PASS | OF_WATCH_PASS | 
			OF_AVRUP_PASS | OF_AVRLOWER_PASS);
		SETPASS(OF_POWERFAIL_PASS);
		SETACT(ACF_POWERFAIL);
		evtlog.new_msg(msg_info, TXT_INF_POWERFAIL);
	}
	// Standby mode 
	if (ALLOK(US_STANDBY, OF_STANDBY_PASS, US_STANDBY)) {
		flag_clear(&appcfg.ulOpFlag, OF_POWERON_PASS | OF_POWERFAIL_PASS | OF_WATCH_PASS | 
			OF_BATTERYLOW_PASS | OF_AVRLOWER_PASS | OF_AVRLOWER_PASS);
		SETPASS(OF_STANDBY_PASS);
		SETACT(ACF_STANDBY);
		evtlog.new_msg(msg_info, TXT_INF_STANDBY);
	}
	// Battery low
	if (ALLOK(US_BATTERYLOW, OF_BATTERYLOW_PASS, US_BATTERYLOW)) {
		SETPASS(OF_BATTERYLOW_PASS);
		SETACT(ACF_BATTERYLOW);
		evtlog.new_msg(msg_info, TXT_INF_BATTERYLOW);
	}
	// AVR low
	RESETIFNOMORESET(US_AVRLOWER, OF_AVRLOWER_PASS, US_AVRLOWER);
	if (ALLOK(US_AVRLOWER, OF_AVRLOWER_PASS, US_AVRLOWER)) {
		flag_clear(&appcfg.ulOpFlag, OF_AVRUP_PASS);
		SETPASS(OF_AVRLOWER_PASS);
		SETACT(ACF_AVRLOWER);
		evtlog.new_msg(msg_info, TXT_INF_AVRLOWER);
	}
	// AVR up
	RESETIFNOMORESET(US_AVRUP, OF_AVRUP_PASS, US_AVRUP);
	if (ALLOK(US_AVRUP, OF_AVRUP_PASS, US_AVRUP)) {
		flag_clear(&appcfg.ulOpFlag, OF_AVRLOWER_PASS);
		SETPASS(OF_AVRUP_PASS);
		SETACT(ACF_AVRUP);
		evtlog.new_msg(msg_info, TXT_INF_AVRUP);
	}
	// Bd moduw
	RESETIFNOMORESET(US_BADEXTERNALMODULES, OF_BADEXTERNALMODULES_PASS, US_BADEXTERNALMODULES);
	if (ALLOK(US_BADEXTERNALMODULES, OF_BADEXTERNALMODULES_PASS, US_BADEXTERNALMODULES)) {
		SETPASS(OF_BADEXTERNALMODULES_PASS);
		SETACT(ACF_BADEXTERNALMODULES);
		evtlog.new_msg(msg_info, TXT_INF_BADEXTERNALMODULES);
	}
	// Internal error
	RESETIFNOMORESET(US_INTERNALERROR, OF_INTERNALERROR_PASS, US_INTERNALERROR);
	if (ALLOK(US_INTERNALERROR, OF_INTERNALERROR_PASS, US_INTERNALERROR)) {
		SETPASS(OF_INTERNALERROR_PASS);
		SETACT(ACF_INTERNALERROR);
		evtlog.new_msg(msg_info, TXT_INF_INTERNALERROR);
	}
	// Inverter failure
	RESETIFNOMORESET(US_INVERTERFAILURE, OF_INVERTERFAILURE_PASS, US_INVERTERFAILURE);
	if (ALLOK(US_INVERTERFAILURE, OF_INVERTERFAILURE_PASS, US_INVERTERFAILURE)) {
		SETPASS(OF_INVERTERFAILURE_PASS);
		SETACT(ACF_INVERTERFAILURE);
		evtlog.new_msg(msg_info, TXT_INF_INVERTERFAILURE);
	}
	// BYPASS
	if (ALLOK(US_BYPASS, OF_BYPASS_PASS, US_BYPASS)) {
		SETPASS(OF_BYPASS_PASS);
		SETACT(ACF_BYPASS);
		evtlog.new_msg(msg_info, TXT_INF_BYPASS);
	}
	// OVERLOAD
	RESETIFNOMORESET(US_OVERLOAD, OF_OVERLOAD_PASS, US_OVERLOAD);
	if (ALLOK(US_OVERLOAD, OF_OVERLOAD_PASS, US_OVERLOAD)) {
		SETPASS(OF_OVERLOAD_PASS);
		SETACT(ACF_OVERLOAD);
		evtlog.new_msg(msg_info, TXT_INF_OVERLOAD);
	}
	// SHORT
	RESETIFNOMORESET(US_SHORT, OF_SHORT_PASS, US_SHORT);
	if (ALLOK(US_SHORT, OF_SHORT_PASS, US_SHORT)) {
		SETPASS(OF_SHORT_PASS);
		SETACT(ACF_SHORT);
		evtlog.new_msg(msg_info, TXT_INF_SHORT);
	}
	// TEMPERATURETOOHIGH
	RESETIFNOMORESET(US_TEMPERATURETOOHIGH, OF_TEMPERATURETOOHIGH_PASS, US_TEMPERATURETOOHIGH);
	if (ALLOK(US_TEMPERATURETOOHIGH, OF_TEMPERATURETOOHIGH_PASS, US_TEMPERATURETOOHIGH)) {
		flag_clear(&appcfg.ulOpFlag, OF_TEMPERATURETOOLOW_PASS);
		SETPASS(OF_TEMPERATURETOOHIGH_PASS);
		SETACT(ACF_TEMPERATURETOOHIGH);
		evtlog.new_msg(msg_info, TXT_INF_TEMPERATURETOOHIGH);
	}
	// TEMPERATURETOOLOW
	RESETIFNOMORESET(US_TEMPERATURETOOLOW, OF_TEMPERATURETOOLOW_PASS, US_TEMPERATURETOOLOW);
	if (ALLOK(US_TEMPERATURETOOLOW, OF_TEMPERATURETOOLOW_PASS, US_TEMPERATURETOOLOW)) {
		flag_clear(&appcfg.ulOpFlag, OF_TEMPERATURETOOHIGH_PASS);
		SETPASS(OF_TEMPERATURETOOLOW_PASS);
		SETACT(ACF_TEMPERATURETOOLOW);
		evtlog.new_msg(msg_info, TXT_INF_TEMPERATURETOOLOW);
	}
	// Watch mode 
	if (ALLOK(US_WATCH, OF_WATCH_PASS, US_WATCH)) {
		flag_clear(&appcfg.ulOpFlag, OF_POWERON_PASS | OF_POWERFAIL_PASS | OF_STANDBY_PASS);
		SETPASS(OF_WATCH_PASS);
		SETACT(ACF_WATCH);
		evtlog.new_msg(msg_info, TXT_INF_WATCH);
	}
	// if battery depleted then show a message 
	if (ALLOK(US_BATTERYDEPLETED, OF_BATTERYDEPLETED_PASS, US_BATTERYDEPLETED)) {
		SETPASS(OF_BATTERYDEPLETED_PASS);
		SETACT(ACF_BATTERYDEPLETED);
		evtlog.new_msg(msg_info, TXT_INF_BATTERYDEPLETEDSH);
	}
	return 1;
}


/*
 * Check if Ups is currently connected.
 */
int testUpsLink(void)
{
	if (drvfn(IOCTL_TESTUPSLINK, NULL, NULL) != IOCTL_ERROR_SUCCESS)
		return 0;
	else
		return 1;
}

/*
 * Main monitoring routine
 */
void monitor(void *lpv)
{
	int iErrorCounterShd=0;
	int iGaudRet=0;

	appcfg.iErrorCounter = 0;
	afctl.iMonitorStatus = STATUS_ACTIVE;
	while(afctl.bMonitorActive)
	{
		sleep(INT_MONITOR_INTERVAL);
		
		iGaudRet=getAllUpsData();
		if (iGaudRet==1) {
			if (appcfg.iErrorCounter > 0)
				appcfg.iErrorCounter--;
		}
		GETAPPFLOWCTL;
		if (!afctl.bMonitorActive) {
			continue;
		}
	
		checkUpsStates();

		UPDATEAPPCFG;
		GETAPPFLOWCTL;

		if (appcfg.iErrorCounter >= INT_MAX_COMMERRORS) {
			evtlog.new_msg(msg_critical, TXT_ERR_TOOMANYCOMMERRORS, 0x01201);
			appcfg.iErrorCounter = 0;
/*
			error_report(TXT_ERR_TOOMANYCOMMERRORS, 0x01201);
			break;
*/
		}
	}
	afctl.iMonitorStatus = STATUS_HASSTOPPED;	
}

void *actions(void *lpv)
{
#undef ISSET
#undef SETACT
#undef UNSETACT
#define ISSET(c, d)	(flag_check(c, d))
#define SETACT(d)		(flag_set(&appcfg.ulActFlag, d))
#define UNSETACT(d)	(flag_clear(&appcfg.ulActFlag, d))
	
	// tablica identyfikatorow zdarzen dla akcji 
	unsigned long ulIds[ INT_MAX_PROGRAMEVENTS ] = {
		ACF_POWERON, ACF_POWERFAIL, ACF_STANDBY, ACF_OVERLOAD, ACF_AVRUP, 
		ACF_AVRLOWER, ACF_BATTERYLOW, ACF_BATTERYDEPLETED, ACF_BYPASS, 
		ACF_WATCH, ACF_SHORT, ACF_INTERNALERROR, ACF_INVERTERFAILURE, 
		ACF_TEMPERATURETOOHIGH, ACF_TEMPERATURETOOLOW, ACF_BADEXTERNALMODULES 
	};
	unsigned long ulActFlagTmp;
	
	sleep(2);

	afctl.iActionStatus = STATUS_ACTIVE;
	while(afctl.bActionActive) 
	{
		//console_report("actions loop...\n");
		
		sleep(INT_ACTION_INTERVAL);

		ulActFlagTmp=appcfg.ulActFlag;
		
		// testujemy flagi akcji do wykonania zdarzen
		for (int ie=0; ie<INT_MAX_PROGRAMEVENTS; ie++) {
			if (!ISSET(ulActFlagTmp, ulIds[ ie ]))
				continue;
			
			UNSETACT(ulIds[ ie ]);
			//console_report("checking for configured actions...\n");
			for (int ic=0; ic<usrcfg.iPeCount; ic++) {
				if (!ISSET(ulIds[ ie ], usrcfg.spe[ic].ulId))
					continue;
				
				if (usrcfg.spe[ic].bEnabled) {
					executeActions(ic);
				}
			}
			//console_report("checking for configured actions done!\n");
		}
	}
	
	afctl.iActionStatus = STATUS_HASSTOPPED;
	pthread_exit(NULL);
}


// Function name	: executeActions
// Description		: Wykonuje reakcje na zdarzenia
// Return type		: void 
// Argument         : int iEventIdx
void executeActions(int iEventIdx)
{
	int iReturn, ilc;

	if (!usrcfg.spe[ iEventIdx ].bNetwork)
	{
		if (usrcfg.spe[iEventIdx].sa.smsg.bEnabled)
		{
#ifdef _DEBUGMODE
			console_report("Action: Message\n");
#endif//_DEBUGMODE
			if (!wall_msg(usrcfg.spe[iEventIdx].sa.smsg.szContent)) {
				evtlog.new_msg(msg_error, TXT_ERR_FAILTOMESSAGE, errno);
			}
		}
		if (usrcfg.spe[iEventIdx].sa.scmd.bEnabled)
		{
#ifdef _DEBUGMODE			
			console_report("Action: Execute\n");
#endif//_DEBUGMODE
			if (system(usrcfg.spe[iEventIdx].sa.scmd.szPath)==-1) {
				evtlog.new_msg(msg_error, TXT_ERR_FAILTOEXECUTE, -1);
			}
		}
		if (usrcfg.spe[iEventIdx].sa.sem.bEnabled)
		{
#ifdef _DEBUGMODE			
			console_report("Action: Email\n");
#endif//_DEBUGMODE
			CEmail ceml;
			if (ceml.Init(usrcfg.szSMTPServer, "", "", "")==0) 
			{
				ceml.SetAuthorization(usrcfg.szSMTPAuth_Name, usrcfg.szSMTPAuth_Password);
				ceml.SetRecipient(usrcfg.spe[iEventIdx].sa.sem.szRecipient);
				ceml.SetSender(usrcfg.szSenderAddress, usrcfg.spe[iEventIdx].sa.sem.szSender);
				if (ceml.SendMessage(usrcfg.spe[iEventIdx].sa.sem.szSubject, 
					usrcfg.spe[iEventIdx].sa.sem.szContent, 
					strlen(usrcfg.spe[iEventIdx].sa.sem.szContent))==0) 
				{
					evtlog.new_msg(msg_info, TXT_INF_MAILSENDTO, 
						usrcfg.spe[iEventIdx].sa.sem.szRecipient);
				}
				else
				{
					char szErrBuff[256] = "";
					ceml.GetLastErrorString(szErrBuff, sizeof(szErrBuff));
					evtlog.new_msg(msg_error, TXT_ERR_FAILTOSENDMAILTO, 
						usrcfg.spe[iEventIdx].sa.sem.szRecipient, szErrBuff);
				}
			} else
				evtlog.new_msg(msg_error, TXT_ERR_FAILTOINITMAILENGINE);
		}
		if (usrcfg.spe[iEventIdx].sa.ssd.bEnabled)
		{
#ifdef _DEBUGMODE			
			console_report("Action: Shutdown\n");
#endif//_DEBUGMODE
			// cancel all pending shutdown threads
			GETAPPFLOWCTL;
			afctl.bShutdownProcActive=false;
			UPDATEAPPFLOWCTL;
			sleep(1);
			
			// initialize new shutdown thread
			GETAPPFLOWCTL;
			afctl.bShutdownProcActive=true;
			UPDATEAPPFLOWCTL;
			
			if (usrcfg.spe[iEventIdx].sa.ssd.ulTimeoutSec==0)
			{
				if (usrcfg.ulUpsShutdown)
				{
					GETAPPFLOWCTL;
					afctl.bUpsOperationActive=true;
					UPDATEAPPFLOWCTL;
#ifdef _DEBUGMODE
					console_report("UPS shutdown!\n");
#endif//_DEBUGMODE
					GETAPPFLOWCTL;
					afctl.bUpsShutdownNow=true;
					UPDATEAPPFLOWCTL;
					for (ilc=0; ilc<30; ilc++) {
						if (!afctl.bUpsShutdownNow)
							break;
						else {
							usleep(500000);
						}
					}
#ifdef _DEBUGMODE
					console_report("UPS shutdown -> done\n!");
#endif//_DEBUGMODE
				}
				GETAPPFLOWCTL;
				afctl.bUpsOperationActive=false;
				UPDATEAPPFLOWCTL;
				if (afctl.bShutdownProcActive) {
					switch(usrcfg.spe[iEventIdx].sa.ssd.esm)
					{
					case sm_normal:
						evtlog.new_msg(msg_info, TXT_INF_SHUTDOWN_NORMAL);
						shutdown_system(sm_normal);
						break;
					
					case sm_reboot:
						evtlog.new_msg(msg_info, TXT_INF_SHUTDOWN_REBOOT);
						shutdown_system(sm_reboot);
						break;
					
					default:
						evtlog.new_msg(msg_warning, TXT_ERR_UNKNOWNSHUTDOWNCONFIG);
						shutdown_system(sm_normal);
						break;
					}
				}//bShutdownTmActive
			}
			else
			{
				int icpid;
				ssdownhlp sbuff;
				sbuff.ulTriggerFlag=usrcfg.spe[ iEventIdx ].ulId;
				sbuff.lpssd = &usrcfg.spe[ iEventIdx ].sa.ssd;
				 
				if (pthread_create(&pthShutdown, NULL, shutdown, &sbuff) > 0) {
					error_report(TXT_ERR_CREATESHUTDOWNPROC);
					evtlog.new_msg(msg_error, TXT_ERR_CREATESHUTDOWNPROC);
				}
			}

		}
	}
	else	// obsluga sieciowych polecen
	{
		sendToNetClients(usrcfg.spe[iEventIdx]);
	}
}

void *shutdown(void/*ssdownhlp*/ *lpsc)
{
#undef ISSET
#define ISSET(c, d)	(flag_check(c, d))

	char szShutdownText[512];
	int ilc;
	unsigned long ulTriggerId = ((ssdownhlp *)lpsc)->ulTriggerFlag;
	lpsshutdown lpss = ((ssdownhlp *)lpsc)->lpssd;
	unsigned long ulTimeOut=time(NULL) + lpss->ulTimeoutSec;

	if (lpss->ulTimeoutSec <= 300)
		sprintf(szShutdownText, TXT_EVENT_PREPARETOSHUTDOWN_SEC, lpss->ulTimeoutSec);
	else if ((lpss->ulTimeoutSec > 300) && (lpss->ulTimeoutSec<1800))
		sprintf(szShutdownText, TXT_EVENT_PREPARETOSHUTDOWN_MIN, lpss->ulTimeoutSec / 60);
	else
		sprintf(szShutdownText, TXT_EVENT_PREPARETOSHUTDOWN_HRS, lpss->ulTimeoutSec / 3600);

	wall_msg(szShutdownText);

	while(afctl.bShutdownProcActive) 
	{
		if (time(NULL) < ulTimeOut) {
			usleep(500000);
		} else {
			if (usrcfg.ulUpsShutdown)
			{
				GETAPPFLOWCTL;
				afctl.bUpsOperationActive=true;
				UPDATEAPPFLOWCTL;
#ifdef _DEBUG
				console_report("UPS shutdown!\n");
#endif//_DEBUG
					GETAPPFLOWCTL;
					afctl.bUpsShutdownNow=true;
					UPDATEAPPFLOWCTL;
					for (ilc=0; ilc<30; ilc++) {
						if (afctl.bUpsShutdownNow==false)
							break;
						else
							sleep(1);
					}
#ifdef _DEBUG
					console_report("UPS shutdown -> OK!\n");
#endif//_DEBUG
			}
			GETAPPFLOWCTL;
			afctl.bUpsOperationActive=false;
			UPDATEAPPFLOWCTL;

			if (afctl.bShutdownProcActive) {
				switch(lpss->esm)
				{
				case sm_normal:
					evtlog.new_msg(msg_info, TXT_INF_SHUTDOWN_NORMAL);
					shutdown_system(sm_normal);
					break;
				
				case sm_reboot:
					evtlog.new_msg(msg_info, TXT_INF_SHUTDOWN_REBOOT);
					shutdown_system(sm_reboot);
					break;
				
				default:
					evtlog.new_msg(msg_warning, TXT_ERR_UNKNOWNSHUTDOWNCONFIG);
					shutdown_system(sm_normal);
					break;
				}
			}//bShutdownTmActive
			GETAPPFLOWCTL;
			afctl.bShutdownProcActive=false;
			UPDATEAPPFLOWCTL;
			return NULL;
		}
	}
	if (!afctl.bShutdownProcActive) {
		wall_msg(TXT_EVENT_PREPARETOSHUTDOWNABORT);
	}
	pthread_exit(NULL);
}

int shutdown_system(int imode)
{
#ifdef _DEBUGMODE
	wall_msg("SHUTDOWN HAS BEEN STARTED!!!!\n");
#else
	switch(imode)
	{
	default:
	case sm_normal:
		wall_msg(TXT_INF_SHUTDOWN_NORMAL);
		evtlog.new_msg(msg_info, TXT_INF_SHUTDOWN_NORMAL);
		system(usrcfg.szShutdownCmd);
		break;
		
	case sm_reboot:
		wall_msg(TXT_INF_SHUTDOWN_REBOOT);
		evtlog.new_msg(msg_info, TXT_INF_SHUTDOWN_REBOOT);
		system("reboot");
		break;
	}

	afctl.bActionActive=afctl.bMonitorActive=false;
	sleep(2);
	terminate(0);
	
#endif
	return 1;
}

// Function name	: sendToNetClients
// Description		: sends info to the network clients
int sendToNetClients(sprogramevent spe)
{
	u_short usPort=12321;
	char szNetBuff[2048], szNetBuff1[2048], szNetBuff2[2048];
	struct hostent *lphentry;
	struct sockaddr_in sai;
	int iLen, iRet;
	int fNetSock;
	unsigned long ulAddr;
	bool bDomainName;
	pthread_t pthNet;

	if (!spe.bNetwork)
		return 0;	// hmmm...cos nie tak???

	if (spe.sa.ssd.ulTimeoutSec > 0) {
		if (spe.sa.ssd.bEnabled) {
			bNetShutdownTmActive=true;
			pthread_create(&pthNet, NULL, netShutdownThread, &spe);
		}
	}
	for (int i=0; i<spe.iNetCompsIdsCnt; i++)
	{
		bDomainName=isalpha(*usrcfg.snc[ spe.ulNetCompIds[ i ] ].szComputer);
		ulAddr=inet_addr(usrcfg.snc[ spe.ulNetCompIds[ i ] ].szComputer);

		if (bDomainName)
			lphentry=gethostbyname(usrcfg.snc[ spe.ulNetCompIds[ i ] ].szComputer);
		else
			lphentry=gethostbyaddr((const char *)&ulAddr, 4, AF_INET);

		if (lphentry==NULL)
			continue;
		
		sai.sin_family=AF_INET;
		sai.sin_addr=*((in_addr*)*lphentry->h_addr_list);
		sai.sin_port=htons(usPort);
		
		fNetSock=socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
		if (fNetSock==-1) {
			evtlog.new_msg(msg_error, TXT_ERR_CREATESOCKET, errno);
			return -1;
		}
		bzero(szNetBuff, sizeof(szNetBuff));
		sprintf(szNetBuff, APP_NET_MAGIC, APP_NET_VERSION_MAJOR, APP_NET_VERSION_MINOR);

		/* komunikat */
		if (spe.sa.smsg.bEnabled) {
			bzero(szNetBuff1, sizeof(szNetBuff));
			bzero(szNetBuff2, sizeof(szNetBuff));
			strcpy(szNetBuff1, szNetBuff);
			iLen=strlen(szNetBuff);
			sprintf(szNetBuff2, APP_NET_CMDFMT, APP_NET_MAGIC01, APP_NET_MAGIC02, APP_NET_CMD_SHOWMESSAGE);
			iLen += APP_NET_CMDFMT_LEN;
			strcat(szNetBuff2, spe.sa.smsg.szContent);
			iLen += strlen(spe.sa.smsg.szContent);
			szNetBuff2[ iLen++ ] = '\0'; szNetBuff2[ iLen++ ] = '\0';

			strcat(szNetBuff1, szNetBuff2);
			iRet=sendto(fNetSock, szNetBuff1, iLen, 0, (sockaddr *)&sai, sizeof(sockaddr));
			if (iRet==-1) {
				evtlog.new_msg(msg_error, TXT_ERR_SENDDATATOCLIENT, usrcfg.snc[ spe.ulNetCompIds[ i ] ].szComputer, errno);
				//close(fNetSock);
			}
		} 
		/* polecenie systemowe */
		if (spe.sa.scmd.bEnabled) {
			bzero(szNetBuff1, sizeof(szNetBuff));
			bzero(szNetBuff2, sizeof(szNetBuff));
			strcpy(szNetBuff1, szNetBuff);
			iLen=strlen(szNetBuff);
			sprintf(szNetBuff2, APP_NET_CMDFMT, APP_NET_MAGIC01, APP_NET_MAGIC02, APP_NET_CMD_EXECUTE);
			iLen += APP_NET_CMDFMT_LEN;
			
			strcat(szNetBuff2, spe.sa.scmd.szPath);
			iLen += strlen(spe.sa.scmd.szPath);
			strcat(szNetBuff2, "\t");
			iLen++;
			strcat(szNetBuff2, spe.sa.scmd.szDirectory);
			iLen += strlen(spe.sa.scmd.szDirectory);
			strcat(szNetBuff2, "\t");
			iLen++;
			strcat(szNetBuff2, spe.sa.scmd.szParams);
			iLen += strlen(spe.sa.scmd.szParams);
			szNetBuff2[ iLen++ ] = '\0';
			szNetBuff2[ iLen++ ] = '\0';

			strcat(szNetBuff1, szNetBuff2);
			iRet=sendto(fNetSock, szNetBuff1, iLen, 0, (sockaddr *)&sai, sizeof(sockaddr));
			if (iRet==-1) {
				evtlog.new_msg(msg_error, TXT_ERR_SENDDATATOCLIENT, usrcfg.snc[ spe.ulNetCompIds[ i ] ].szComputer, errno);
				//close(fNetSock);
			}
		}
		// shutdown
		if (spe.sa.ssd.bEnabled && !spe.sa.ssd.ulTimeoutSec) {
			bzero(szNetBuff1, sizeof(szNetBuff));
			bzero(szNetBuff2, sizeof(szNetBuff));
			strcpy(szNetBuff1, szNetBuff);
			iLen=strlen(szNetBuff);
			switch(spe.sa.ssd.esm) {
			case sm_normal:
				sprintf(szNetBuff2, APP_NET_CMDFMT, APP_NET_MAGIC01, APP_NET_MAGIC02, APP_NET_CMD_DOWN);
				break;
			case sm_forced:
				sprintf(szNetBuff2, APP_NET_CMDFMT, APP_NET_MAGIC01, APP_NET_MAGIC02, APP_NET_CMD_DOWNFORCE);
				break;
			case sm_reboot:
				sprintf(szNetBuff2, APP_NET_CMDFMT, APP_NET_MAGIC01, APP_NET_MAGIC02, APP_NET_CMD_DOWNREBOOT);
				break;
			case sm_hibernate:
				sprintf(szNetBuff2, APP_NET_CMDFMT, APP_NET_MAGIC01, APP_NET_MAGIC02, APP_NET_CMD_HIBERNATE);
				break;
			default:
				continue;
			}
			iLen += APP_NET_CMDFMT_LEN;
			strcat(szNetBuff1, szNetBuff2);
			iRet=sendto(fNetSock, szNetBuff1, iLen, 0, (sockaddr *)&sai, sizeof(sockaddr));
			if (iRet==-1) {
				evtlog.new_msg(msg_error, TXT_ERR_SENDDATATOCLIENT, usrcfg.snc[ spe.ulNetCompIds[ i ] ].szComputer, errno);
				//close(fNetSock);
			}
		}
		shutdown(fNetSock, SHUT_RDWR);
		close(fNetSock);
	}
	return 1;
}

// Function name	: NetShutdownThread
void *netShutdownThread(void *lpv)
{
#undef ISSET
#define ISSET(c, d)	(TestBit(c, d))

	lpsprogramevent lpspe = (lpsprogramevent)lpv;
	
	u_short usPort=12321;
	char szNetBuff[2048], szNetBuff1[2048], szNetBuff2[2048];
	struct hostent *lphentry;
	struct sockaddr_in sai;
	int iLen, iRet;
	int fNetSock;
	unsigned long ulAddr;
	bool bDomainName;
	unsigned long ulTimeOut=time(NULL) + lpspe->sa.ssd.ulTimeoutSec;

	while(bNetShutdownTmActive)
	{
		if (time(NULL) < ulTimeOut) {
			//if (!ISSET(sac.ulOpFlag, dwTriggerId))
			//	bNetShutdownTmActive=FALSE;
			usleep(500000);
		}
		else
		{
			bNetShutdownTmActive=false;
			
			for (int i=0; i<lpspe->iNetCompsIdsCnt; i++)
			{
				bDomainName=isalpha(*usrcfg.snc[ lpspe->ulNetCompIds[ i ] ].szComputer);
				ulAddr=inet_addr(usrcfg.snc[ lpspe->ulNetCompIds[ i ] ].szComputer);

				if (bDomainName)
					lphentry=gethostbyname(usrcfg.snc[ lpspe->ulNetCompIds[ i ] ].szComputer);
				else
					lphentry=gethostbyaddr((const char *)&ulAddr, 4, AF_INET);

				if (lphentry==NULL)
					continue;
				
				sai.sin_family=AF_INET;
				sai.sin_addr=*((in_addr *)*lphentry->h_addr_list);
				sai.sin_port=htons(usPort);
				fNetSock=socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
				if (fNetSock==-1) {
					evtlog.new_msg(msg_error, TXT_ERR_CREATESOCKET, errno);
					continue;
				}
				sprintf(szNetBuff, APP_NET_MAGIC, APP_NET_VERSION_MAJOR, APP_NET_VERSION_MINOR);
				iLen=strlen(szNetBuff);
				// shutdown
				if (lpspe->sa.ssd.bEnabled)
				{
					strcpy(szNetBuff1, szNetBuff);
					iLen=strlen(szNetBuff);
					switch(lpspe->sa.ssd.esm) {
					case sm_normal:
						sprintf(szNetBuff2, APP_NET_CMDFMT, APP_NET_MAGIC01, APP_NET_MAGIC02, APP_NET_CMD_DOWN);
						break;
					case sm_forced:
						sprintf(szNetBuff2, APP_NET_CMDFMT, APP_NET_MAGIC01, APP_NET_MAGIC02, APP_NET_CMD_DOWNFORCE);
						break;
					case sm_reboot:
						sprintf(szNetBuff2, APP_NET_CMDFMT, APP_NET_MAGIC01, APP_NET_MAGIC02, APP_NET_CMD_DOWNREBOOT);
						break;
					case sm_hibernate:
						sprintf(szNetBuff2, APP_NET_CMDFMT, APP_NET_MAGIC01, APP_NET_MAGIC02, APP_NET_CMD_HIBERNATE);
						break;
					default:
						continue;
					}
					iLen += APP_NET_CMDFMT_LEN;
					strcat(szNetBuff1, szNetBuff2);
					iRet=sendto(fNetSock, szNetBuff1, iLen, 0, (sockaddr *)&sai, sizeof(sockaddr));
					if (iRet==-1) {
						evtlog.new_msg(msg_error, TXT_ERR_SENDDATATOCLIENT, usrcfg.snc[ lpspe->ulNetCompIds[ i ] ].szComputer, errno);
						close(fNetSock);
					}
					shutdown(fNetSock, SHUT_RDWR);
					close(fNetSock);
				}
			}
		}
	}
}

/* read main configuration */
int update_main_config()
{
	char *fp=get_config_filepath(APP_FILE_MAINCONFIG);
	if (fp==NULL) {
		error_report("%s (%s)", TXT_ERR_MEMALLOC, strerror(errno));
		return -1;
	}
	pCfgMain=new CConf;
	if (pCfgMain->init(fp)!=CONF_SUCCESS) {
		free(fp);
		evtlog.new_msg(msg_error, "%s (%s)", TXT_ERR_READAPPCONFIG, pCfgMain->geterrordesc());
		error_report("%s (%s)", TXT_ERR_READAPPCONFIG, pCfgMain->geterrordesc());
		if (pCfgMain!=NULL)
			delete pCfgMain;
		return -1;
	} else {
		free(fp);
		pCfgMain->parse_config();
	}
	
	if (compile_main_config_map(*pCfgMain)==-1) {
		evtlog.new_msg(msg_error, "%s", TXT_ERR_READAPPCONFIG);
		error_report("%s", TXT_ERR_READAPPCONFIG);
		if (pCfgMain!=NULL)
			delete pCfgMain;
		return -1;
	} else {
		if (pCfgMain!=NULL)
			delete pCfgMain;
	}
	
	return 0;	// success
}

/* read user configuration */
int update_user_config()
{
	if (pCfgUser!=NULL)
		delete pCfgUser;
	
	char *fp=get_config_filepath(str_ext_config);
	if (fp==NULL) {
		error_report("%s (%s)", TXT_ERR_MEMALLOC, strerror(errno));
		return -1;
	}
	pCfgUser=new CConf;
	if (pCfgUser->init(fp)!=CONF_SUCCESS) {
		free(fp);
		evtlog.new_msg(msg_error, "%s (%s)", TXT_ERR_READUSERCONFIG, pCfgUser->geterrordesc());
		error_report("%s (%s)", TXT_ERR_READUSERCONFIG, pCfgUser->geterrordesc());
		delete pCfgUser;
		return -1;
	} else
		pCfgUser->parse_config();
		
	free(fp);

	if (compile_user_config_map(*pCfgUser)==-1) {
		evtlog.new_msg(msg_error, "%s", TXT_ERR_READUSERCONFIG);
		error_report("%s", TXT_ERR_READUSERCONFIG);
		delete pCfgUser;
		return -1;
	}
	
	//
	// user actions config
	//
	fp=get_config_filepath(appcfg.sdrvcfg.szCfgFileName);
	if (fp==NULL) {
		error_report("%s (%s)", TXT_ERR_MEMALLOC, strerror(errno));
		delete pCfgUser;
		return -1;
	}
	CConf *pCfgEv=new CConf;
	if (pCfgEv->init(fp)!=CONF_SUCCESS) {
		free(fp);
		evtlog.new_msg(msg_error, "%s", TXT_ERR_READUSERCONFIG);
		error_report("%s", TXT_ERR_READUSERCONFIG);
		delete pCfgEv;
		delete pCfgUser;
		return -1;
	} else {
		free(fp);
		pCfgEv->parse_config();
	}
	
	const int iNoOfItems=3;
	const char szCfgItems[iNoOfItems][128] = { "de%02d.id\0", "de%02d.name\0", "de%02d.action.index\0" };
	char szBuf[128] = "", *s;
	int iMaxItems=INT_MAX_PROGRAMEVENTS, iEventIdx=0;
	
	for (int iItem=0; iItem<iMaxItems; iItem++) {
		for (int i=0; i<iNoOfItems; i++) {
			sprintf(szBuf, szCfgItems[i], iItem);
			s=pCfgEv->getcfgitemvalue((char *)szBuf);
			if (s!=NULL && s!="") {
				switch(i) {
				case 0:
					usrcfg.spe[iEventIdx].ulId=atol(s); 
					break;
				case 1:
					strcpy(usrcfg.spe[iEventIdx].szTitle, s); 
					break;
				case 2: {
						int iActIdx=atol(s);
						if (read_action_config(pCfgUser, iActIdx, &usrcfg.spe[iEventIdx])==0)	// on success
							usrcfg.spe[iEventIdx].bEnabled=true;
						else
							usrcfg.spe[iEventIdx].bEnabled=false;
					}
					break;
				}// switch(i)
			}// if (s!=NULL && s!="")
			if (i<2) {
				if (s==NULL || s=="")
					break;
				if (usrcfg.spe[iEventIdx].ulId==0) {
					usrcfg.spe[iEventIdx].bEnabled=false;
					break;
				}
			} else if (i==2 && s!=NULL && s!="")
				iEventIdx++;
		}
	}
	usrcfg.iPeCount=iEventIdx;
 	delete pCfgEv;

	//
	// user network clients config
	//	
	fp=get_config_filepath(APP_FILE_NETCONFIG);
	if (fp==NULL) {
		error_report("%s (%s)", TXT_ERR_MEMALLOC, strerror(errno));
		delete pCfgUser;
		return -1;
	}
	CConf *pCfgNet=new CConf;
	if (pCfgNet->init(fp)!=CONF_SUCCESS) {
		free(fp);
		evtlog.new_msg(msg_error, "%s", TXT_ERR_READUSERCONFIG);
		error_report("%s", TXT_ERR_READUSERCONFIG);
		delete pCfgNet;
		delete pCfgUser;
		return -1;
	} else {
		free(fp);
		pCfgNet->parse_config();
	}
	//
	// network computers
	//
	usrcfg.iNcCount=0;
	for (int iItem=0; iItem<INT_MAX_NETCOMPUTERS; iItem++) {
		sprintf(szBuf, "nc%02d.name", iItem);
		s=pCfgNet->getcfgitemvalue((char *)szBuf);
		if (s!=NULL && strlen(s)) {
			strcpy(usrcfg.snc[usrcfg.iNcCount].szComputer, s);
			usrcfg.snc[usrcfg.iNcCount++].bEnabled=true;
		} else {
			continue;
		}
	}
	
 	delete pCfgNet;
	delete pCfgUser;
	
	return 0;	// success
}

int read_action_config(CConf *pCfg, int iIndex, sprogramevent *lpspe)
{
	const int iNoOfItems=16;
	const char szCfgItems[iNoOfItems][128] = { 
		"ua%02d.alert.enable\0", "ua%02d.alert.text\0", 
		"ua%02d.command.enable\0", "ua%02d.command.path\0", "ua%02d.command.directory\0", "ua%02d.command.params\0",
		"ua%02d.email.enable\0", "ua%02d.email.sender\0", "ua%02d.email.recipient\0", "ua%02d.email.subject\0", "ua%02d.email.content\0",
		"ua%02d.shutdown.enable\0", "ua%02d.shutdown.reboot\0", "ua%02d.shutdown.timeout\0", 
		"ua%02d.network.enable\0", "ua%02d.network.clients\0" };
	char szBuf[128], *s;
	
	for (int i=0; i<iNoOfItems; i++) {
		sprintf(szBuf, szCfgItems[i], iIndex);
		s=pCfg->getcfgitemvalue((char *)szBuf);
		if (s!=NULL && strlen(s)) {
			switch(i) {
			case 0:
				lpspe->sa.smsg.bEnabled = (bool)atoi(s);
				break;
			case 1:
				strcpy(lpspe->sa.smsg.szContent, s);
				break;
			case 2:
				lpspe->sa.scmd.bEnabled = (bool)atoi(s);
				break;
			case 3:
				strcpy(lpspe->sa.scmd.szPath, s);
				break;
			case 4:
				strcpy(lpspe->sa.scmd.szDirectory, s);
				break;
			case 5:
				strcpy(lpspe->sa.scmd.szParams, s);
				break;
			case 6:
				lpspe->sa.sem.bEnabled = (bool)atoi(s);
				break;
			case 7:
				strcpy(lpspe->sa.sem.szSender, s);
				break;
			case 8:
				strcpy(lpspe->sa.sem.szRecipient, s);
				break;
			case 9:
				strcpy(lpspe->sa.sem.szSubject, s);
				break;
			case 10:
				strcpy(lpspe->sa.sem.szContent, s);
				break;
			case 11:
				lpspe->sa.ssd.bEnabled = (bool)atoi(s);
				break;
			case 12:
				if ((bool)atoi(s))
					lpspe->sa.ssd.esm=sm_reboot;
				else
					lpspe->sa.ssd.esm=sm_normal;
				break;
			case 13:
				lpspe->sa.ssd.ulTimeoutSec = (unsigned long)strtoul(s, NULL, 10);
				break;
			case 14:
				lpspe->bNetwork = (bool)atoi(s);
				break;
			case 15:
				{
					lpspe->iNetCompsIdsCnt=0;
					char *tok=strtok(s, ",;");
					for (int icl=0; icl<INT_MAX_NETCOMPUTERS; icl++) {
						if (tok!=NULL && tok!="")
							lpspe->ulNetCompIds[lpspe->iNetCompsIdsCnt++]=atoi(tok);
						else
							break;
						tok=strtok(NULL, ",;");
					}
				}
				break;			
			}
		} else
			continue;
	}
	return 0;	// success
}

// Description : Ta funkcja wypelnia tablice laczenia parametrow zasilacza 
//             : uzywajac do tego celu parametrow przekazanych do funkcji.
//             : Tablica laczaca lpspb musi miec rozmiar INT_MAX_PROGRAMPARAMS.
//             : Wartosc zwracana to INT_MAX_PROGRAMPARAMS lub 0 dla bledu.
int fill_params_struct( lpsparams_binder lpspb, lpsups_parameters lpsup )
{
	int iItem = 0;
	if ( !lpspb || !lpsup )
		return 0;

	lpspb[ iItem ].ulId = UP_UID_FAMILY;
	lpspb[ iItem ].bIntegral = 0;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_IDENTIFIER_FAMILY );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_IDENTIFIER_FAMILY_DSC );
	lpspb[ iItem ].lpDivPtr = NULL;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->sz__uid_family;

	lpspb[ iItem ].ulId = UP_UID_MODEL;
	lpspb[ iItem ].bIntegral = 0;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_IDENTIFIER_MODEL );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_IDENTIFIER_MODEL_DSC );
	lpspb[ iItem ].lpDivPtr = NULL;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->sz__uid_model;

	lpspb[ iItem ].ulId = UP_UID_VERSION;
	lpspb[ iItem ].bIntegral = 0;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_IDENTIFIER_VERSION );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_IDENTIFIER_VERSION_DSC );
	lpspb[ iItem ].lpDivPtr = NULL;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->sz__uid_version;

	lpspb[ iItem ].ulId = UP_BATTERYSTATE;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_BATTERYSTATE );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_BATTERYSTATE_DSC );
	lpspb[ iItem ].lpDivPtr = NULL;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__batterystate;

	lpspb[ iItem ].ulId = UP_VLT_BATTERY;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_VOLTAGE_BATT );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_VOLTAGE_BATT_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__vlt_battery_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__vlt_battery;

	lpspb[ iItem ].ulId = UP_PRC_BATTERYCAPACITY;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_PERCENT_BATTCAP );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_PERCENT_BATTCAP_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__prc_batterycapacity_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__prc_batterycapacity;

	lpspb[ iItem ].ulId = UP_VLT_BATTERYLOAD;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_VOLTAGE_BATTLOAD );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_VOLTAGE_BATTLOAD_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__vlt_batteryload_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__vlt_batteryload;

	lpspb[ iItem ].ulId = UP_VLT_INPUT;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_VOLTAGE_INPUT );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_VOLTAGE_INPUT_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__vlt_input_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__vlt_input;

	lpspb[ iItem ].ulId = UP_FRQ_INPUT;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_FREQUENCY_INPUT );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_FREQUENCY_INPUT_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__frq_input_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__frq_input;

	lpspb[ iItem ].ulId = UP_VLT_OUTPUT;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_VOLTAGE_OUTPUT );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_VOLTAGE_OUTPUT_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__vlt_output_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__vlt_output;

	lpspb[ iItem ].ulId = UP_FRQ_OUTPUT;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_FREQUENCY_OUTPUT );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_FREQUENCY_OUTPUT_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__frq_output_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__frq_output;

	lpspb[ iItem ].ulId = UP_CUR_OUTPUT;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_CUR_OUTPUT );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_CUR_OUTPUT_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__cur_output_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__cur_output;

	/* TODO - remove it. It's currently replaced by other power values */
	lpspb[ iItem ].ulId = UP_LOAD_OUTPUT;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_LOAD_OUTPUT );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_LOAD_OUTPUT_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__load_output_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__load_output;

	lpspb[ iItem ].ulId = UP_PWR_APPARENT;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_PWR_APPARENT );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_PWR_APPARENT_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__pwr_apparent_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__pwr_apparent;

	lpspb[ iItem ].ulId = UP_PWR_EFFECTIVE;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_PWR_EFFECTIVE );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_PWR_EFFECTIVE_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__pwr_effective_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__pwr_effective;

	lpspb[ iItem ].ulId = UP_PWR_REACTIVE;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_PWR_REACTIVE );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_PWR_REACTIVE_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__pwr_reactive_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__pwr_reactive;

	lpspb[ iItem ].ulId = UP_PWR_COEFFICIENT;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_PWR_COEFFICIENT );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_PWR_COEFFICIENT_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__pwr_coefficient_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__pwr_coefficient;

	lpspb[ iItem ].ulId = UP_PRC_POWERWEIGHT;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_PERCENT_POWERWEIGHT );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_PERCENT_POWERWEIGHT_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__prc_powerweight_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__prc_powerweight;

	lpspb[ iItem ].ulId = UP_TMV_AUTONOMY;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_TIME_AUTONOMY );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_TIME_AUTONOMY_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__tmv_autonomy_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__tmv_autonomy;

	lpspb[ iItem ].ulId = UP_TMV_ONBATTERY;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_TIME_ONBATTERY );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_TIME_ONBATTERY_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__tmv_onbattery_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__tmv_onbattery;

	lpspb[ iItem ].ulId = UP_TMP_INVERTER;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_TEMP_INVERTER );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_TEMP_INVERTER_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__tmp_inverter_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__tmp_inverter;

	lpspb[ iItem ].ulId = UP_TMP_CASING;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_TEMP_CASING );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_TEMP_CASING_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__tmp_casing_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__tmp_casing;

	lpspb[ iItem ].ulId = UP_TMP_BATTERY;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_TEMP_BATTERY );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_TEMP_BATTERY_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__tmp_battery_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__tmp_battery;

	lpspb[ iItem ].ulId = UP_OV_EXTMODULES;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_EXTMODULES );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_EXTMODULES_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__ov_extmodules_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__ov_extmodules;

	lpspb[ iItem ].ulId = UP_PST_UPPER;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_POWERSTATE_UPPER );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_POWERSTATE_UPPER_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__pst_upper_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__pst_upper;

	lpspb[ iItem ].ulId = UP_PST_LOWER;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_POWERSTATE_LOWER );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_POWERSTATE_LOWER_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__pst_lower_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__pst_lower;

	lpspb[ iItem ].ulId = UP_PST_AVRUP;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_POWERSTATE_AVRUP );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_POWERSTATE_AVRUP_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__pst_avrup_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__pst_avrup;

	lpspb[ iItem ].ulId = UP_PST_AVRLOW;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_POWERSTATE_AVRLOW );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_POWERSTATE_AVRLOW_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__pst_avrlow_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__pst_avrlow;

	lpspb[ iItem ].ulId = UP_SNS_VOLTAGE;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_SENSITIVITY_VOLTAGE );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_SENSITIVITY_VOLTAGE_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__sns_voltage_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__sns_voltage;

	lpspb[ iItem ].ulId = UP_SNS_FREQUENCY;
	lpspb[ iItem ].bIntegral = 1;
	strcpy( lpspb[ iItem ].szName, TXT_PARAM_SENSITIVITY_FREQUENCY );
	strcpy( lpspb[ iItem ].szDesc, TXT_PARAM_SENSITIVITY_FREQUENCY_DSC );
	lpspb[ iItem ].lpDivPtr = &lpsup->ui__sns_frequency_div;
	lpspb[ iItem++ ].lpVarPtr = &lpsup->ui__sns_frequency;

	return iItem;
}

// Function name	: getUpsStateString
// Description		: Zwraca status aktualnego trybu pracy w postaci treci komunikatu
int getUpsStateString(char *lpszBuff, unsigned long ulMask, unsigned long ulState, bool bAttention)
{
	int i;
	if (!lpszBuff || !ulMask)
		return -1;
	/* third parameter gives ability to help differentiate the types of events */
	sbinder sbevent[ INT_MAX_PROGRAMEVENTS ] = {
		{ US_POWERON, TXT_EVENT_POWERON, 0 }, { US_POWERFAIL, TXT_EVENT_POWERFAIL, 0 }, 
		{ US_STANDBY, TXT_EVENT_STANDBY, 0 }, { US_OVERLOAD, TXT_EVENT_OVERLOAD, 1 },
		{ US_AVRUP, TXT_EVENT_AVRUP, 1 }, { US_AVRLOWER, TXT_EVENT_AVRLOWER, 1 }, 
		{ US_BATTERYLOW, TXT_EVENT_BATTERYLOW, 1 }, { US_BATTERYDEPLETED, TXT_EVENT_BATTERYDEPLETED, 1 }, 
		{ US_BYPASS, TXT_EVENT_BYPASS, 0 }, { US_WATCH, TXT_EVENT_WATCH, 0 }, 
		{ US_SHORT, TXT_EVENT_SHORT, 1 }, { US_INTERNALERROR, TXT_EVENT_INTERNALERROR, 1 }, 
		{ US_INVERTERFAILURE, TXT_EVENT_INVERTERFAILURE, 1 },
		{ US_TEMPERATURETOOHIGH, TXT_EVENT_TEMPERATURETOOHIGH, 1 },
		{ US_TEMPERATURETOOLOW, TXT_EVENT_TEMPERATURETOOLOW, 1 }, 
		{ US_BADEXTERNALMODULES, TXT_EVENT_BADEXTERNALMODULES, 1 }
	};

	if (!flag_check(ulMask, ulState))
		return 0;
	if (!bAttention)
		strcpy(lpszBuff, TXT_EVENT_UNKNOWNSTATE);
	else
		strcpy(lpszBuff, "");
	for (i = 0; i < INT_MAX_PROGRAMEVENTS; i++) {
		if ((ulState & sbevent[i].ulId) && (bAttention==(bool)sbevent[i].ulFlag)) {
			strcpy(lpszBuff, sbevent[i].szText);
			break;
		}
	}
	if (i >= INT_MAX_PROGRAMEVENTS)
		return 0;
	return 1;
}


void print_device_parameters(lpsapp_config lpsac)
{
	char szBatteryState[5][64]={
		TXT_PARAM_BS_CHARGING,TXT_PARAM_BS_FULL,TXT_PARAM_BS_DISCHARGING,
		TXT_PARAM_BS_DISCHARGED,TXT_PARAM_BS_DEPLETED};
	unsigned int *uiValTmp, *uiDivTmp;
	char szTmpVal[256];
	lpsparams_binder lpspb = new sparams_binder[INT_MAX_PROGRAMPARAMS];
	
	if (lpspb!=NULL) {
		printf("===========================================================\n");
		/* status */
		printf(TXT_DESCPARAM_CURRENTSTATE);
		for (int ispc=0; ispc<36-strlen(TXT_DESCPARAM_CURRENTSTATE); ispc++) printf(" ");
		if (getUpsStateString(szTmpVal, lpsac->supsi.ulUpsStateMask, lpsac->supsi.ulUpsState, 0)==1) {
			printf(": %s\n", szTmpVal);
		} else {
			printf(": %s\n", TXT_EVENT_UNKNOWNSTATE);
		}
		/* alarm */
		printf(TXT_DESCPARAM_CURRENTALARM);
		for (int ispc=0; ispc<36-strlen(TXT_DESCPARAM_CURRENTALARM); ispc++) printf(" ");
		if (getUpsStateString(szTmpVal, lpsac->supsi.ulUpsStateMask, lpsac->supsi.ulUpsState, 1)==1) {
			printf(": %s\n", szTmpVal);
		} else {
			printf(": - \n");
		}
		
		if (fill_params_struct(lpspb, &lpsac->supsi.sUpsParameters) < INT_MAX_PROGRAMPARAMS)
			return;
		printf("-----------------------------------------------------------\n");
		for (int i = 0; i < INT_MAX_PROGRAMPARAMS; i++) {
			if (!flag_check(lpsac->supsi.sUpsParameters.ulMask, lpspb[i].ulId))
				continue;
			if (lpspb[i].bIntegral) {
				uiValTmp = (unsigned int *)(lpspb[i].lpVarPtr);
				uiDivTmp = (unsigned int *)(lpspb[i].lpDivPtr);
				if (uiDivTmp != NULL)
					if ((*uiDivTmp != 1) && (*uiDivTmp != -1))
						sprintf(szTmpVal, "%.1f", ((float)(*uiValTmp) / (float)(*uiDivTmp)));
					else
						sprintf(szTmpVal, "%.0f", ((float)(*uiValTmp) / (float)(*uiDivTmp)));
				else
					if (lpspb[i].ulId == UP_BATTERYSTATE)
						sprintf(szTmpVal, "%s", (char *)szBatteryState[(*uiValTmp)]);
					else
						sprintf(szTmpVal, "%d", *uiValTmp);
			} else {
				if (lpspb[i].lpVarPtr!=NULL)
					sprintf(szTmpVal, "%s", (char *)lpspb[i].lpVarPtr);
			}
			printf("%s", lpspb[i].szName);
			if (strlen(lpspb[i].szName)<36)
				for (int ispc=0; ispc<36-strlen(lpspb[i].szName); ispc++) 
					printf(" ");
			printf(": %s ", szTmpVal);
			if (lpspb[i].bIntegral && lpspb[i].ulId!=UP_BATTERYSTATE)
				printf("[%s]\n", lpspb[i].szDesc);
			else
				printf("\n");
		}
		delete [] lpspb;
		printf("===========================================================\n");
	} else {
		console_report("%s (%s)\n", TXT_ERR_MEMALLOC, strerror(errno));
	}
}

int service_start()
{
	CEmail ceml;
	unsigned long ulBuff;
	int icnt, iReturn=0;	// buffer for return values
	pid_t actpid;		// action process pid
	key_t smkey;
	
	signal(SIGSEGV, terminate);
	signal(SIGINT, terminate);
	signal(SIGQUIT, terminate);
	signal(SIGTERM, terminate);

	evtlog.open(APP_FILE_EVENTLOG, APP_PSCORE);
	
	if (update_main_config()!=0) {
		goto __cleanup__;
	}

	if ((drvfn=get_ups_interface(appcfg.sdrvcfg.szName))==NULL) {
		evtlog.new_msg(msg_error, "%s", TXT_ERR_DRIVERREAD);
		console_report("%s", TXT_ERR_DRIVERREAD);
		goto __cleanup__;
	}
	
	ulBuff=sizeof(appcfg.sdrvcfg);
	iReturn=drvfn(IOCTL_GET_DRIVER_INFO, (void *)&appcfg.sdrvcfg, (int *)&ulBuff);
	if (iReturn!=IOCTL_ERROR_SUCCESS) {
		evtlog.new_msg(msg_error, TXT_ERR_GETDRIVERINFO);
		console_report(TXT_ERR_GETDRIVERINFO"\n");
		goto __cleanup__;
	}
	
	evtlog.new_msg(msg_info, TXT_SERVICE_STARTING);
	
	/* driver init */
	ulBuff=2;	// force comm initialization, shutdown if error
	iReturn=drvfn(IOCTL_INIT, &ulBuff, NULL);
	if (iReturn==IOCTL_ERROR_NOTYETCONFIGURED) {
		iReturn=drvfn(IOCTL_AUTOCONFIGURE, NULL, NULL);
		if (iReturn!=IOCTL_ERROR_SUCCESS) {
			bDrvInitialized=false;
			iReturn=drvfn(IOCTL_GET_ERRORNO, NULL, NULL);
			evtlog.new_msg(msg_error, TXT_ERR_DRIVERNOTCONFIGURED, iReturn);
			console_report(TXT_ERR_DRIVERNOTCONFIGURED"\n", iReturn);
			goto __cleanup__;
		} else {
			iReturn=drvfn(IOCTL_UNINIT, NULL, NULL);
			iReturn=drvfn(IOCTL_INIT, NULL, NULL);
		}
	}
	if (iReturn!=IOCTL_ERROR_SUCCESS) {
		iReturn=drvfn(IOCTL_GET_ERRORNO, NULL, NULL);
		evtlog.new_msg(msg_error, TXT_ERR_DRIVERINITFAIL, -1, iReturn);
		console_report(TXT_ERR_DRIVERINITFAIL"\n", -1, iReturn);
		bDrvInitialized=false;
		iReturn=drvfn(IOCTL_UNINIT, NULL, NULL);
		goto __cleanup__;
	} else
		bDrvInitialized=true;
	
	ulBuff=sizeof(appcfg.supsi.ulMask);
	iReturn=drvfn(IOCTL_GET_UPSINFOMASK, (void *)&appcfg.supsi.ulMask, (int *)&ulBuff);
	if (iReturn!=IOCTL_ERROR_SUCCESS) {
		iReturn=drvfn(IOCTL_GET_ERRORNO, NULL, NULL);
		evtlog.new_msg(msg_error, TXT_ERR_DRIVERINITFAIL, 0x01008, iReturn);
		console_report(TXT_ERR_DRIVERINITFAIL"\n", 0x01008, iReturn);
		goto __cleanup__;
	}
	
	if (update_main_config()!=0) {
		goto __cleanup__;
	}
	/* update user configuration */
	if (update_user_config()!=0) {
		goto __cleanup__;
	}
	
	appcfg.ulActFlag=0;
	appcfg.ulOpFlag=0;
	appcfg.iErrorCounter=0;
	
	/* start */
	afctl.bMonitorActive = afctl.bActionActive = true;
	afctl.iMonitorStatus = afctl.iActionStatus = STATUS_INACTIVE;
	
	create_pid_file(APP_PSCORE);
	
	// create dummy shared mem - applicable always when system has no 
	// shared memory segments initialized, because shmem with id equal to 0 is created 
	// which is, in some way failing to work correctly
	{
		CShMem *pcDummy = new CShMem;
		if (pcDummy!=NULL) {
			if (pcDummy->create(0,1)!=SHM_FAILURE) {
				sleep(1);
				pcDummy->close();
			}
			delete pcDummy;
		}
	}
		
	// the real chared memory segment
	pShmMain=new CShMem;
	if (pShmMain->createkey(SHM_PATHNAME, SHM_PROJID)!=SHM_SUCCESS) {
		console_report("%s (%s)\n",TXT_ERR_SHAREMEMCREATE, strerror(errno));
		goto __cleanup__;
	} else {
		if (pShmMain->create(iShmSize)!=SHM_SUCCESS) {
			console_report("%s (%s)\n",TXT_ERR_SHAREMEMCREATE, strerror(errno));
			if (pShmMain->open(iShmSize)!=SHM_SUCCESS) {
				goto __cleanup__;
			}
		}
		//console_report("SHM Created: shmid=%d; sem_id=%d\n", pShmMain->shm_id, pShmMain->sem_id);
	}
	UPDATEAPPCFG;
	UPDATEUSRCFG;
	UPDATEAPPFLOWCTL;
	
	iReturn=drvfn(IOCTL_TESTUPSLINK, NULL, NULL);
	if (iReturn!=IOCTL_ERROR_SUCCESS) {
		iReturn=drvfn(IOCTL_GET_ERRORNO, NULL, NULL);
		evtlog.new_msg(msg_error, TXT_ERR_UPSLINKFAIL, 0x01009, iReturn);
		console_report("%s", TXT_ERR_UPSLINKFAILSH);
		goto __cleanup__;
	} else {
		evtlog.new_msg(msg_info, TXT_ERR_UPSLINKOK, 0x0100A);
	}
	// Test if ups state monitoring is available and if not, just end the game :P
	if (!flag_check(appcfg.supsi.ulMask, UI_UPS_STATE)) {
		evtlog.new_msg(msg_critical, TXT_ERR_NOSTATEMONITORING, 0x0100B);
		goto __cleanup__;
	}

	if (flag_check(appcfg.supsi.ulMask, UI_CHARACTERISTICS)) {
		ulBuff=sizeof(appcfg.supsi.sUpsCharacteristic);
		iReturn=drvfn(IOCTL_GET_UPSCHARCTERISTIC, 
			(void *)&appcfg.supsi.sUpsCharacteristic, (int *)&ulBuff);
		if (iReturn!=IOCTL_ERROR_SUCCESS)
			evtlog.new_msg(msg_error, TXT_ERR_GETUPSPARAMS, 0x0100C);
	}
	
	/* first contact */
	getAllUpsData();
	checkUpsStates();
	
	print_device_parameters(&appcfg);
	
	/* create another form of life */
	pthread_create(&pthAction, NULL, actions, (void *)(icnt=0));
	monitor(NULL);

	stop_monitoring();

__cleanup__:
	
	if (bDrvInitialized) {
		iReturn=drvfn(IOCTL_UNINIT, NULL, NULL);
		if (iReturn != IOCTL_ERROR_SUCCESS)
			evtlog.new_msg(msg_error, TXT_ERR_DRIVERUNINITFAIL, 0x01021);
		else
		    bDrvInitialized = false;
	}
	terminate(-1);
	
	return 0;
}

void stop_monitoring()
{
	const int timeout_sec = 10;
	
	if (afctl.iActionStatus!=STATUS_INACTIVE) {
		/* wait for action thread */
		if (afctl.bActionActive)
			afctl.bActionActive=false;
		int icnt=timeout_sec;
		while(afctl.iActionStatus!=STATUS_HASSTOPPED) {
			sleep(1);
			if ((icnt--)<=0)
				break;
		}
	}
	
	if (afctl.iMonitorStatus!=STATUS_INACTIVE) {
		/* wait for monitor */
		if (afctl.bMonitorActive)
			afctl.bMonitorActive=false;
		int icnt=timeout_sec;
		while(afctl.iMonitorStatus!=STATUS_HASSTOPPED) {
			sleep(1);
			if ((icnt--)<=0)
				break;
		}
	}
	
	if (bDrvInitialized) {
		int iReturn=drvfn(IOCTL_UNINIT, NULL, NULL);
		if (iReturn != IOCTL_ERROR_SUCCESS)
			evtlog.new_msg(msg_error, TXT_ERR_DRIVERUNINITFAIL, 0x01021);
		else
		    bDrvInitialized = false;
	}
}

void terminate(int iCode)
{
	rem_pid_file(APP_PSCORE);
	
	if (iCode!=-1)
		stop_monitoring();
	
	if (pShmMain!=NULL) {
		pShmMain->close();
		delete pShmMain;
	}
	if (iCode>-1) {
		evtlog.new_msg(msg_info, TXT_ERR_TERMINATION, iCode);
	}
	if (iCode>=-1) {
		evtlog.new_msg(msg_info, TXT_ERR_SERVICESTOPPED);
		console_report("\n%s\n", TXT_ERR_SERVICESTOPPED);
	}
	evtlog.close();
	if (iCode>-1) {
		exit(0);
	}
}
