/***************************************************************************
 *   Copyright (C) 2004 by EVER Sp. z o.o.                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef _STRUCTS_H_
#define _STRUCTS_H_

#include "gldefs.h"

/* 
 * Ups states
 */
#define	US_POWERON					0x00000001
#define	US_POWERFAIL				0x00000002
#define	US_STANDBY					0x00000004
#define	US_OVERLOAD					0x00000008	/* assume last active state */
#define	US_AVRUP					0x00000010	/* also us_poweron */
#define	US_AVRLOWER					0x00000020	/* also us_poweron */
#define	US_BATTERYLOW				0x00000040	/* also us_powerfail */
#define	US_BATTERYDEPLETED			0x00000080	/* battery depleted - assume any state before */
#define	US_BYPASS					0x00000100
#define	US_WATCH					0x00000200	/* online (but not visible) load batteries only */
#define	US_SHORT					0x00000400
#define	US_INTERNALERROR			0x00000800
#define	US_INVERTERFAILURE			0x00001000
#define	US_TEMPERATURETOOHIGH		0x00002000
#define	US_TEMPERATURETOOLOW		0x00004000
#define	US_BADEXTERNALMODULES		0x00008000

/* 
 * Ups parameters
 */
#define	UP_PST_UPPER				0x00000001
#define	UP_PST_LOWER				0x00000002
#define	UP_PST_AVRUP				0x00000004
#define	UP_PST_AVRLOW				0x00000008
#define	UP_FRQ_INPUT				0x00000010
#define	UP_FRQ_OUTPUT				0x00000020
#define	UP_VLT_INPUT				0x00000040
#define	UP_VLT_OUTPUT				0x00000080
#define	UP_VLT_BATTERYLOAD			0x00000100
#define	UP_VLT_BATTERY				0x00000200
#define	UP_PRC_BATTERYCAPACITY		0x00000400
#define	UP_PRC_POWERWEIGHT			0x00000800
#define	UP_SNS_VOLTAGE				0x00001000
#define	UP_SNS_FREQUENCY			0x00002000
#define	UP_TMV_AUTONOMY				0x00004000
#define	UP_TMV_ONBATTERY			0x00008000
#define	UP_TMP_INVERTER				0x00010000
#define	UP_TMP_CASING				0x00020000
#define	UP_TMP_BATTERY				0x00040000
#define	UP_UID_FAMILY				0x00080000
#define	UP_UID_VERSION				0x00100000
#define	UP_UID_MODEL				0x00200000
#define	UP_LOAD_OUTPUT				0x00400000
#define	UP_OV_EXTMODULES			0x00800000
#define	UP_BATTERYSTATE				0x01000000
#define UP_PWR_EFFECTIVE			0x02000000	// moc czynna
#define UP_PWR_APPARENT				0x04000000	// moc pozorna
#define UP_PWR_REACTIVE				0x08000000	// moc bierna
#define UP_CUR_OUTPUT				0x10000000	// prd wyjciowy
#define UP_PWR_COEFFICIENT			0x20000000	// wspczynnik mocy

/* 
 * Ups setup parameters
 */
/* power state switch tresholds */
#define	UP_SET_PST_UPPER				0x00000001
#define	UP_SET_PST_LOWER				0x00000002
#define	UP_SET_PST_AVRUP				0x00000004
#define	UP_SET_PST_AVRLOW				0x00000008
/* audible alarms */
#define	UP_SET_AAL_ON					0x00000010
/* timers */
#define	UP_SET_TMR_POWERFAILTOSTANDBY	0x00000020
#define	UP_SET_TMR_PROGRAM_UP			0x00000040
#define	UP_SET_TMR_PROGRAM_DOWN			0x00000080
/* frequency */
#define	UP_SET_FRQ_INPUT				0x00000100
#define	UP_SET_FRQ_OUTPUT				0x00000200
/* sensitivities */
#define	UP_SET_SNS_VOLTAGE				0x00000400
#define	UP_SET_SNS_FREQUENCY			0x00000800
/* ups identification */
#define	UP_SET_UID_USERNAME				0x00001000

/* 
 * Ups characteristics fields flags
 */
#define	UC_POWEROUT_MAX					0x00000001
#define	UC_ACCUSCALE					0x00000002
#define	UC_ADDMOD_MAXCAP				0x00000004
#define	UC_PST_LOWER_MIN				0x00000008
#define	UC_PST_LOWER_MAX				0x00000010
#define	UC_PST_UPPER_MIN				0x00000020
#define	UC_PST_UPPER_MAX				0x00000040
#define	UC_PST_AVRUP_MIN				0x00000080
#define	UC_PST_AVRDOWN_MAX				0x00000100
#define	UC_INFREQ_MIN					0x00000200
#define	UC_INFREQ_MAX					0x00000400
#define	UC_OUTFREQ_MIN					0x00000800
#define	UC_OUTFREQ_MAX					0x00001000
#define	UC_OUTVOLTAGE					0x00002000
#define	UC_ACCUFULLCAPVOLTAGE			0x00004000

/* 
 * Ups info fields flags
 */
#define	UI_CHARACTERISTICS				0x00000001
#define	UI_PARAMETERS					0x00000002
#define	UI_SETUP_PARAMETERS				0x00000004
#define	UI_UPS_STATE					0x00000008

/* 
 * Main program operational flags
 */
#define OF_POWERON_PASS					US_POWERON
#define OF_POWERFAIL_PASS				US_POWERFAIL
#define OF_STANDBY_PASS					US_STANDBY
#define OF_OVERLOAD_PASS				US_OVERLOAD
#define OF_AVRUP_PASS					US_AVRUP
#define OF_AVRLOWER_PASS				US_AVRLOWER
#define OF_BATTERYLOW_PASS				US_BATTERYLOW
#define OF_BATTERYDEPLETED_PASS			US_BATTERYDEPLETED
#define OF_BYPASS_PASS					US_BYPASS
#define OF_WATCH_PASS					US_WATCH
#define OF_SHORT_PASS					US_SHORT
#define OF_INTERNALERROR_PASS			US_INTERNALERROR
#define OF_INVERTERFAILURE_PASS			US_INVERTERFAILURE
#define OF_TEMPERATURETOOHIGH_PASS		US_TEMPERATURETOOHIGH
#define OF_TEMPERATURETOOLOW_PASS		US_TEMPERATURETOOLOW
#define OF_BADEXTERNALMODULES_PASS		US_BADEXTERNALMODULES

/* 
 * Main program action/reaction flags
 */
#define ACF_POWERON					US_POWERON
#define ACF_POWERFAIL				US_POWERFAIL
#define ACF_STANDBY					US_STANDBY
#define ACF_OVERLOAD				US_OVERLOAD
#define ACF_AVRUP					US_AVRUP
#define ACF_AVRLOWER				US_AVRLOWER
#define ACF_BATTERYLOW				US_BATTERYLOW
#define ACF_BATTERYDEPLETED			US_BATTERYDEPLETED
#define ACF_BYPASS					US_BYPASS
#define ACF_WATCH					US_WATCH
#define ACF_SHORT					US_SHORT
#define ACF_INTERNALERROR			US_INTERNALERROR
#define ACF_INVERTERFAILURE			US_INVERTERFAILURE
#define ACF_TEMPERATURETOOHIGH		US_TEMPERATURETOOHIGH
#define ACF_TEMPERATURETOOLOW		US_TEMPERATURETOOLOW
#define ACF_BADEXTERNALMODULES		US_BADEXTERNALMODULES
#define ACF_ALL						( ACF_POWERON | ACF_POWERFAIL | ACF_STANDBY | ACF_OVERLOAD | \
										ACF_AVRUP | ACF_AVRLOWER | ACF_BATTERYLOW | \
										ACF_BATTERYDEPLETED | ACF_BYPASS | ACF_WATCH | ACF_SHORT | \
										ACF_INTERNALERROR | ACF_INVERTERFAILURE | \
										ACF_TEMPERATURETOOHIGH | ACF_TEMPERATURETOOLOW | \
										ACF_BADEXTERNALMODULES )

/* 
 * Action status flags
 */
#define ACSF_DONE					0x0001

/* Application internal flags */
#define APPF_LINKLOSTSTATE			0x0001

/* 
 * Ups communication type enumerator
 */
typedef enum _eups_link_
{
	ul_serial,
	ul_parallel,
	ul_usb,
	ul_irda,
	ul_ethernet,
	ul_other
} eups_link;

/* 
 * System shutdown modes
 */
typedef enum _eshutdown_mode_
{
	sm_normal,
	sm_forced,
	sm_reboot,
	sm_hibernate
} eshutdown_mode;

/*
 * Battery states
 */
typedef enum _ebattery_state
{
	bs_charging,
	bs_full,
	bs_discharging,
	bs_discharged,
	bs_depleted
} ebattery_state;

/* 
 * Ups characteristics structure
 */
typedef struct _sups_characteristic_
{
	unsigned long	ulSize;
	/* mask used to determine which fields of this struct. are used */
	unsigned long	ulMask;

	unsigned int	ui__powerout_max;
	unsigned int	ui__powerout_max_div;
	
	unsigned int	ui__accuscale;
	unsigned int	ui__accuscale_div;
	
	unsigned int	ui__addmod_maxcap;
	unsigned int	ui__addmod_maxcap_div;
	
	unsigned int	ui__pst_lower_min;
	unsigned int	ui__pst_lower_min_div;
	unsigned int	ui__pst_lower_max;
	unsigned int	ui__pst_lower_max_div;
	unsigned int	ui__pst_upper_min;
	unsigned int	ui__pst_upper_min_div;
	unsigned int	ui__pst_upper_max;
	unsigned int	ui__pst_upper_max_div;
	unsigned int	ui__pst_avrup_min;
	unsigned int	ui__pst_avrup_min_div;
	unsigned int	ui__pst_avrdown_max;
	unsigned int	ui__pst_avrdown_max_div;

	unsigned int	ui__infreq_min;
	unsigned int	ui__infreq_min_div;
	unsigned int	ui__infreq_max;
	unsigned int	ui__infreq_max_div;
	unsigned int	ui__outfreq_min;
	unsigned int	ui__outfreq_min_div;
	unsigned int	ui__outfreq_max;
	unsigned int	ui__outfreq_max_div;

	unsigned int	ui__outvoltage;
	unsigned int	ui__outvoltage_div;

	unsigned int	ui__accufullcapvoltage;
	unsigned int	ui__accufullcapvoltage_div;

} sups_characteristic, supsch, *lpsups_characteristic, *lpsupsch;

/* 
 * Ups parameters structure
 */
typedef struct _sups_parameters_
{
	unsigned long	ulSize;
	/* mask used to determine which fields of this struct. are used */
	unsigned long	ulMask;
	/* power state switch tresholds */
	unsigned int	ui__pst_upper;
	unsigned int	ui__pst_upper_div;
	unsigned int	ui__pst_lower;
	unsigned int	ui__pst_lower_div;
	unsigned int	ui__pst_avrup;
	unsigned int	ui__pst_avrup_div;
	unsigned int	ui__pst_avrlow;
	unsigned int	ui__pst_avrlow_div;
	/* frequency */
	unsigned int	ui__frq_input;
	unsigned int	ui__frq_input_div;
	unsigned int	ui__frq_output;
	unsigned int	ui__frq_output_div;
	/* voltage variables */
	unsigned int	ui__vlt_input;
	unsigned int	ui__vlt_input_div;
	unsigned int	ui__vlt_output;
	unsigned int	ui__vlt_output_div;
	unsigned int	ui__vlt_batteryload;
	unsigned int	ui__vlt_batteryload_div;
	unsigned int	ui__vlt_battery;
	unsigned int	ui__vlt_battery_div;
	/* percentage values */
	unsigned int	ui__prc_batterycapacity;
	unsigned int	ui__prc_batterycapacity_div;
	unsigned int	ui__prc_powerweight;
	unsigned int	ui__prc_powerweight_div;
	/* sensitivities */
	unsigned int	ui__sns_voltage;
	unsigned int	ui__sns_voltage_div;
	unsigned int	ui__sns_frequency;
	unsigned int	ui__sns_frequency_div;
	/* time values */
	unsigned int	ui__tmv_autonomy;
	unsigned int	ui__tmv_autonomy_div;
	unsigned int	ui__tmv_onbattery;
	unsigned int	ui__tmv_onbattery_div;
	/* temperature */
	unsigned int	ui__tmp_inverter;
	unsigned int	ui__tmp_inverter_div;
	unsigned int	ui__tmp_casing;
	unsigned int	ui__tmp_casing_div;
	unsigned int	ui__tmp_battery;
	unsigned int	ui__tmp_battery_div;
	/* ups identification */
	char				sz__uid_family[64];
	char				sz__uid_version[64];
	char				sz__uid_model[64];
	/* power weights */
	unsigned int	ui__load_output;			/* obciazenie wyjscia */
	unsigned int	ui__load_output_div;
	/* other values */
	unsigned int	ui__ov_extmodules;
	unsigned int	ui__ov_extmodules_div;
	unsigned int	ui__batterystate;			/* 
															status baterii 
															(ladowanie, naladowana, rozladowywanie, 
															rozladowane, do wymiany) 
														*/
	/* moc czynna */
	unsigned int	ui__pwr_effective;
	unsigned int	ui__pwr_effective_div;
	/* moc pozorna */
	unsigned int	ui__pwr_apparent;
	unsigned int	ui__pwr_apparent_div;
	/* moc bierna */
	unsigned int	ui__pwr_reactive;
	unsigned int	ui__pwr_reactive_div;
	/* prd wyjciowy */
	unsigned int	ui__cur_output;
	unsigned int	ui__cur_output_div;
	/* wspczynnik mocy */
	unsigned int	ui__pwr_coefficient;
	unsigned int	ui__pwr_coefficient_div;

} sups_parameters, supsp, *lpsups_parameters, *lpsupsp;

/* 
 * Ups setup parameters structure
 */
typedef struct _sups_setup_parameters_
{
	unsigned long	ulSize;
	/* mask used to determine which fields of this struct. are used */
	unsigned long	ulMask;
	/* power state switch tresholds */
	unsigned int	ui__set_pst_upper;
	unsigned int	ui__set_pst_upper_div;
	unsigned int	ui__set_pst_lower;
	unsigned int	ui__set_pst_lower_div;
	unsigned int	ui__set_pst_avrup;
	unsigned int	ui__set_pst_avrup_div;
	unsigned int	ui__set_pst_avrlow;
	unsigned int	ui__set_pst_avrlow_div;
	/* audible alarms */
	unsigned int	ui__set_aal_on;
	/* timers */
	unsigned int	ui__set_tmr_powerfailtostandby;
	unsigned int	ui__set_tmr_powerfailtostandby_div;
	unsigned int	ui__set_tmr_program_up;
	unsigned int	ui__set_tmr_program_up_div;
	unsigned int	ui__set_tmr_program_down;
	unsigned int	ui__set_tmr_program_down_div;
	/* frequency */
	unsigned int	ui__set_frq_input;
	unsigned int	ui__set_frq_input_div;
	unsigned int	ui__set_frq_output;
	unsigned int	ui__set_frq_output_div;
	/* sensitivities */
	unsigned int	ui__set_sns_voltage;
	unsigned int	ui__set_sns_voltage_div;
	unsigned int	ui__set_sns_frequency;
	unsigned int	ui__set_sns_frequency_div;
	/* ups identification */
	char			sz__set_uid_username[64];

} sups_setup_parameters, supssp, *lpsups_setup_parameters, *lpsupssp;

/* 
 * Main ups information structure
 */
typedef struct _sups_info_
{
	/* structure size - used for version identification */
	unsigned long	ulSize;
	unsigned long	ulMask;
	
	/* flag register used to determine which UPS states are used 
	 * from _ups_state_ enumerator.
	 */
	unsigned long	ulUpsStateMask;
	unsigned long	ulUpsState;

	/* UPS characteristic structure*/
	supsch			sUpsCharacteristic;
	
	/* UPS parameters structure */
	supsp			sUpsParameters;
	
	/* UPS setup parameters structure */
	supssp			sUpsSetupParameters;

} sups_info, *lpsups_info;

/* 
 * Events configuration structure
 */
typedef struct _smgm_eventscfg_
{
	/* identifier of event */
	unsigned long	ulId;
	/* event type */
	unsigned long	ulType;
	/* displayed name of event */
	char				szName[128];
	/* true - if event is signalled */
	bool				bState;
} smgm_eventscfg;

/* 
 * Driver specification structure (RO)
 */
typedef struct _sdrv_info_
{
	/* structure size - used for version identification */
	unsigned long	ulSize;
	/* driver version (Major.Minor) */
	unsigned int	uiVersionMajor;
	unsigned int	uiVersionMinor;
	/* driver class name */
	char			szName[64];
	/* hardware family name */
	char			szFamily[64];
	/* Type of communication medium */
	eups_link		eLink;
	/* Config file name */
	char			szCfgFileName[MAX_PATHBUFF];
	/* UPS bitmap file name */
	char			szBmpFileName[MAX_PATHBUFF];

} sdrv_info, *lpsdrv_info;

/*
 * Message structure
 */
typedef struct _smessage_
{
	/* message is active if this flag is set */
	bool	bEnabled;
	/* message content */
	char	szContent[ 256 ];

} smessage, *lpsmessage;

/* 
 * Command structure
 */
typedef struct _scommand_
{
	/* command is active if this flag is set */
	bool	bEnabled;
	/* command path */
	char	szPath[ MAX_PATHBUFF ];
	/* command directory */
	char	szDirectory[ MAX_PATHBUFF ];
	/* command parameters */
	char	szParams[ MAX_PATHBUFF ];

} scommand, *lpscommand;

/* 
 * E-Mail message structure
 */
typedef struct _semailmsg_
{
	/* email message is active if this flag is set */
	char	bEnabled;
	/* recipient address */
	char	szRecipient[ 256 ];
	/* subject of message */
	char	szSubject[ 128 ];
	/* content of message */
	char	szContent[ 256 ];
	/* sender name = send from */
	char	szSender[ 256 ];

} semailmsg, *lpsemailmsg;

/* 
 * Shutdown structure
 */
typedef struct _sshutdown_
{
	/* shutdown is active if this flag is set */
	bool	bEnabled;
	/* how to shutdown */
	eshutdown_mode	esm;
	/* shutdown after this time (in seconds) */
	unsigned long	ulTimeoutSec;

} sshutdown, *lpsshutdown;

/* 
 * User Action structure
 */
typedef struct _saction_
{
	/* action - message box */
	smessage	smsg;
	/* action - execute command */
	scommand	scmd;
	/* action - send e-mail message */
	semailmsg	sem;
	/* action - shutdown system */
	sshutdown	ssd;
	/* action status flag */
	unsigned long	ulflag;

} saction, *lpsaction;

/* 
 * Network computer structure
 */
typedef struct _snetworkcomputer_
{
	/* is computer enabled */
	bool	bEnabled;
	/* computer name or IP address */
	char	szComputer[256];
} snetworkcomputer, *lpsnetworkcomputer;

/* 
 * Program event structure
 */
typedef struct _sprogramevent_
{
	/* event identifier */
	unsigned long	ulId;
	/* is event enabled */
	bool				bEnabled;
	/* full event name */
	char				szTitle[128];
	/* number of actions to do */
	/* unsigned long	ulActCnt; */
	/* number of network actions to do */
	/* unsigned long	ulActFlag; */
	/* action to do if event occur */
	saction			sa;
	/* network action to do if event occur */
	bool				bNetwork;
	/* number of network computers id's */
	int				iNetCompsIdsCnt;
	/* network computers id's */
	unsigned long	ulNetCompIds[ INT_MAX_NETCOMPUTERS ];

} sprogramevent, *lpsprogramevent;

/* 
 * User configuration
 */
typedef struct _suser_config_
{
	/* structure size */
	unsigned long	ulSize;
	/* do ups shutdown after system shutdown or not */
	unsigned long	ulUpsShutdown;
	/* e-mail parameters - smtp server address */
	char			szSMTPServer[128];
	/* e-mail parameters - sender e-mail address */
	char			szSenderAddress[128];
	/* SMTP authorization - user */
	char			szSMTPAuth_Name[128];
	/* SMTP authorization - password */
	char			szSMTPAuth_Password[128];
	/* shutdown command */
	char			szShutdownCmd[MAX_PATHBUFF];
	/* number of configured program events */
	int			iPeCount;
	/* user - program events */
	sprogramevent	spe[ INT_MAX_PROGRAMEVENTS ];
	/* number of network computers */
	int			iNcCount;
	/* network computers */
	snetworkcomputer	snc[ INT_MAX_NETCOMPUTERS ];

} suser_config, *lpsuser_config;

/* 
 * Configuration universal values for internal use
 */
#undef INT_STRINGVALUE_MAX
#define INT_STRINGVALUE_MAX	256
#undef INT_STRINGLIST_MAX
#define INT_STRINGLIST_MAX	48
#undef INT_LISTITEMS_MAX
#define INT_LISTITEMS_MAX	128
#undef INT_VALUENAME_MAX
#define INT_VALUENAME_MAX	64

/* Enumerator typow*/
typedef enum _ecfg_vtype_
{
	VT_INTEGER,
	VT_LONG,
	VT_STRING,
	VT_INTLIST,
	VT_STRINGPWD,
} ecfg_vtype;

typedef struct tagList{
	char szName[ INT_STRINGLIST_MAX ]; 
	int iValue;
};

typedef struct tagRange{ 
	long lMin; 
	long lMax;
};
	
/* Struktura uniwersalnej wartosci */
typedef struct _scfg_value_
{
	/* type of value */
	ecfg_vtype eType;
	/* value name */
	char szName[ INT_VALUENAME_MAX ];
	/* selected value buffer */
	union { 
		int iValue; long lValue; char szValue[ INT_STRINGVALUE_MAX ]; 
	} value;
	/* value ranges (only for long and integer type) */
	tagRange	range;

	/* output divider */
	double	dDivider;
	/* list values */
	int		iListItems;
	tagList	list[ INT_LISTITEMS_MAX ];

} scfg_value, *lpscfg_value;

/***********************************************************
 * main program structure
 ***********************************************************/
typedef struct _sapp_config_
{
	/* driver selected for communication with UPS */
	sdrv_info	sdrvcfg;
	/* complete informations about ups */
	sups_info	supsi;
	/* error counter */
	int				iErrorCounter;
	/* operational flag register */
	unsigned long	ulOpFlag;
	/* action/reaction flag register */
	unsigned long	ulActFlag;
	/* last configured driver index */
	int				iDriver;

} sapp_config, *lpsapp_config;

/* 
 * Struktura laczaca parametr z konkretnym wskaznikiem do danej
 */
typedef struct _sparams_binder_
{
	/* parameter - identifier */
	unsigned long ulId;
	/* parameter - name */
	char szName[128];
	/* pointer to parameter value */
	void * lpVarPtr;
	/* pointer to value divider */
	void * lpDivPtr;
	/* parameter - desc */
	char szDesc[16];
	/* type of data */
	bool bIntegral;

} sparams_binder, *lpsparams_binder;

/* 
 * Uniwersalna struktura laczaca
 */
typedef struct _sbinder_
{
	/* identifier */
	unsigned long ulId;
	/* text to bind to */
	char szText[256];
	/* flag register */
	unsigned long ulFlag;

} sbinder, *lpsbinder;

/* 
 * Struktura program PSMain
 */
typedef struct _spsmain_config_
{
	/* driver selected for communication with UPS */
	sdrv_info	sdrvcfg;
	/* complete informations about ups */
	sups_info	supsi;

} spsmain_config, *lpspsmain_config;

/*
 * Shutdown structure helper
 * contains trigger and main shutdown structure
 */
typedef struct _ssdownhlp_ {
	lpsshutdown	lpssd;					// main shutdown structure
	unsigned long	ulTriggerFlag;		// shutdown trigger flag
} ssdownhlp, *lpssdownhlp;

/*
 * flow control identifiers
 */
#define STATUS_INACTIVE		0
#define STATUS_ACTIVE		1
#define STATUS_HASSTOPPED	2

/*
 * Program flow-control structure
 */
typedef struct _sflow_ctl_ {
	/* flag used to control monitoring thread */
	bool	bMonitorActive;
	int	iMonitorStatus;	// 0-inactive, 1-active, 2-has stopped
	/* flag used to control action thread */
	bool	bActionActive;
	int	iActionStatus;		// 0-inactive, 1-active, 2-has stopped
	/* flag used to control main event loop */
	bool	bMainLoop;
	/* flag used to determine if debug mode is enabled */
	bool	bDebugMode;
	/* flag used to determine if update to UPS settings is needed */
	bool	bConfigUpdate;
	/* application internal flags */
	bool	bUpsStateReading;
	bool	bUpsOperationActive;
	bool	bShutdownProcActive;
	bool	bNetShutdownProcActive;
	bool	bUpsShutdownNow;
} sflow_ctl, *lpsflow_ctl;


#endif /*_STRUCTS_H_*/

